@@ -1,16 +1,22 @@
 """ Configure and instantiate Celery
 """
-
 import os
+
+from sermos.constants import DEFAULT_RETRY_TASK_MAX_TTL, DEFAULT_MAX_RETRY
+
 if os.environ.get('USE_GEVENT', "False").lower() == 'true':
     from gevent import monkey
     monkey.patch_all()
 
+import random
+import time
+
+from celery_dyrygent.tasks import register_workflow_processor
+
 import sys
 import logging
 from typing import List
 from celery import Celery
-from celery.worker.consumer import Consumer as BaseConsumer
 from sermos.logging_config import setup_logging
 from sermos.utils.module_utils import SermosModuleLoader
 from sermos.utils.task_utils import PipelineGenerator, PipelineResult, \
@@ -20,8 +26,10 @@ from sermos import __version__
 
 logger = logging.getLogger('celery')
 ENABLE_TOOLS = str(os.environ.get('ENABLE_TOOLS', 'false')).lower() == 'true'
+CELERY_TASKS_ACK_LATE = str(os.environ.get('CELERY_TASKS_ACK_LATE', 'false')).lower() == 'true'
 LOG_LEVEL = os.environ.get("LOG_LEVEL", "INFO")
 OVERLOAD_ES = os.environ.get('ENV', 'production').lower() == 'production'
+PIPELINE_CHORD_COMPRESSION = os.environ.get('PIPELINE_CHORD_COMPRESSION', None)
 
 setup_logging(app_version=__version__,
               client_version=sermos_client_version,
@@ -55,6 +63,12 @@ def pipeline_retry(event: dict):
     if gen.good_to_go:
         chain = gen.generate_chain()
         if chain is not None:
+            # Exponential backoff
+            exponential_backoff = min((3 ** gen.pipeline_wrapper.retry_count) +
+                                      (random.randint(0, 1000) / 1000),
+                                      DEFAULT_RETRY_TASK_MAX_TTL)
+            logger.debug(f"Exponential backoff sleep {exponential_backoff}")
+            time.sleep(exponential_backoff)
             # Kick it off again.
             chain.apply_async()
 
@@ -151,59 +165,6 @@ class GenerateCeleryTasks(SermosModuleLoader):
             tmp_handler = self.celery.task(tmp_handler)
 
 
-# def configure_base_celery(celery: Celery = None):
-#     """ Configures the default parameters for a generic Celery instance
-#     """
-#     REDIS_URL = os.environ.get('REDIS_URL', 'redis://localhost:6379/0')
-#     CELERY_BROKER_URL = os.environ.get('CELERY_BROKER_URL', REDIS_URL)
-#     CELERY_RESULT_BACKEND = os.environ.get('CELERY_RESULT_BACKEND', REDIS_URL)
-
-#     if not celery:
-#         celery = Celery('sermos')
-
-#     # Configure the broker and tasks
-#     celery.conf.broker_url = CELERY_BROKER_URL
-
-#     # Reasonable defaults, override as necessary
-#     celery.conf.worker_redirect_stdouts = True
-#     celery.conf.worker_redirect_stdouts_level = LOG_LEVEL
-#     celery.conf.worker_hijack_root_logger = False
-
-#     # NOTE: The broker URL may not be the best result backend. For example,
-#     # When using Rabbit as the broker (recommended), you should use Redis
-#     # as the result backend, as Rabbit has horrible support as backend.
-#     celery.conf.result_backend = CELERY_RESULT_BACKEND
-#     celery.conf.task_ignore_result = False  # Must not ignore for Chords
-#     celery.conf.task_acks_late = False  # Check per worker
-#     celery.conf.result_expires = int(
-#         os.environ.get('CELERY_RESULT_EXPIRES', 10800))  # 3 hours by default
-#     celery.conf.broker_pool_limit = int(os.environ.get('BROKER_POOL_LIMIT',
-#                                                        10))
-#     celery.conf.worker_max_tasks_per_child = int(
-#         os.environ.get('MAX_TASKS_PER_CHILD', 100))
-#     celery.conf.task_soft_time_limit =\
-#         int(os.environ.get('TASK_TIMEOUT_SECONDS', 3600))
-#     celery.conf.task_time_limit =\
-#         int(os.environ.get('TASK_TIMEOUT_SECONDS', 3600)) + 10  # Cleanup buffer
-#     celery.conf.task_serializer = 'json'
-#     celery.conf.result_serializer = 'json'
-#     celery.conf.accept_content = ['json']
-#     # Required config options for some brokers we use frequently.
-#     transport_options = {}
-#     celery.conf.broker_transport_options = transport_options
-
-#     # Sermos generally has long-running tasks (relatively speaking), so
-#     # limit number of jobs a worker can reserve. This may not be true for
-#     # all tasks, so configure this on a per application basis. In the event
-#     # mutltiple task kinds exist in an application (short and long), see
-#     # http://docs.celeryproject.org/en/latest/userguide/optimizing.html#optimizing-prefetch-limit
-#     # for some guidance on combining multiple workers and routing tasks.
-#     # TODO make configurable from env
-#     celery.conf.worker_prefetch_multiplier = 1
-
-#     return celery
-
-
 def configure_celery(celery: Celery):
     """ Configure Sermos-compatible Celery instance. Primarily this means
     compatibility with Pipelines and Scheduled Tasks through injecting the
@@ -221,6 +182,10 @@ def configure_celery(celery: Celery):
         event information easily through a chain.
         """
         abstract = True
+        autoretry_for = (Exception,)
+        max_retries = DEFAULT_MAX_RETRY
+        retry_backoff = True
+        retry_jitter = True
 
         def __call__(self, *args, **kwargs):
             """ Allow the return value of one task to update the kwargs of a
@@ -236,7 +201,11 @@ def configure_celery(celery: Celery):
             # other areas.
             if 'event' not in kwargs.keys():
                 kwargs['event'] = {}
-
+            # This is a special worker from dyrygent that orchestrates our
+            # pipelines.  It provides a patch in fix for celery's poor
+            # implementation of Canvas work-flows
+            if self.__name__ == 'workflow_processor':
+                kwargs.pop('event', None)
             return super(ChainedTask, self).__call__(*args, **kwargs)
 
     celery.Task = ChainedTask
@@ -253,6 +222,9 @@ def configure_celery(celery: Celery):
     celery.conf.worker_redirect_stdouts_level = LOG_LEVEL
     celery.conf.worker_hijack_root_logger = False
 
+    if PIPELINE_CHORD_COMPRESSION:
+      celery.conf.task_compression = PIPELINE_CHORD_COMPRESSION
+
     # NOTE: The broker URL may not be the best result backend. For example,
     # When using Rabbit as the broker (recommended), you should use Redis
     # as the result backend, as Rabbit has horrible support as backend.
@@ -269,6 +241,7 @@ def configure_celery(celery: Celery):
         int(os.environ.get('TASK_TIMEOUT_SECONDS', 3600))
     celery.conf.task_time_limit =\
         int(os.environ.get('TASK_TIMEOUT_SECONDS', 3600)) + 10  # Cleanup buffer
+    celery.conf.task_acks_late = CELERY_TASKS_ACK_LATE
     celery.conf.task_serializer = 'json'
     celery.conf.result_serializer = 'json'
     celery.conf.accept_content = ['json']
@@ -287,6 +260,7 @@ def configure_celery(celery: Celery):
 
     # Add our application's workers & any other tasks to be made
     # available
+    register_workflow_processor(celery)
     try:
         GenerateCeleryTasks(sermos_config, celery).generate()
     except Exception as e:
