@@ -1,8 +1,10 @@
 """ Primary CLI group entrypoint
 """
-import click
 import logging
+import click
+from sermos.logging_config import setup_logging
 
+setup_logging(default_level='INFO', establish_logging_config=False)
 logger = logging.getLogger(__name__)
 
 # Not all CLI tools will be functional / available depending on which extras
@@ -27,4 +29,11 @@ except ImportError as e:
     logger.warning(warning_msg.format("Configuration Server"))
     logger.warning(f"{e}")
 
+try:
+    from sermos.cli.proxy import sermos_proxy
+    collection.append(sermos_proxy)
+except ImportError as e:
+    logger.warning(warning_msg.format("Proxy Service"))
+    logger.warning(f"{e}")
+
 sermos = click.CommandCollection(sources=collection)
