@@ -23,29 +23,31 @@
         print(status)
 
 """
-import json
 import subprocess
 import base64
 import logging
-import requests
-from sermos.utils.module_utils import get_access_key, get_client_pkg_name
+
+from sermos.utils.module_utils import get_client_pkg_name
 from sermos.sermos_yaml import load_sermos_config
-from sermos.constants import DEFAULT_DEPLOY_URL
+from sermos.cloud import SermosCloud
 
 logger = logging.getLogger(__name__)
 
 
-class SermosDeploy():
+class SermosDeploy(SermosCloud):
     """ Primary Sermos Deployment class for invocation and status updates.
     """
     def __init__(self,
+                 deployment_id: str = None,
                  access_key: str = None,
                  pkg_name: str = None,
                  sermos_yaml_filename: str = None,
                  commit_hash: str = None,
-                 deploy_url: str = None,
+                 base_url: str = None,
                  deploy_branch: str = 'main'):
         """ Arguments:
+                deployment_id (optional): UUID for Deployment. Find in your
+                    Sermos Cloud Console.
                 access_key (optional): Access key, issued by Sermos, that
                     dictates the environment into which this request will be
                     deployed. Defaults to checking the environment for
@@ -62,29 +64,21 @@ class SermosDeploy():
                     invocation will be used. This is the default usage, and is
                     useful in the case of a CI/CD pipeline such that the Sermos
                     deployment is invoked after your integration passes.
-                deploy_url (optional): Defaults to primary sermos deployment
-                    endpoint. Only modify this if there is a specific, known
-                    reason to do so.
+                base_url (optional): Defaults to primary Sermos Cloud base URL.
+                    Only modify this if there is a specific, known reason to do so.
+                deploy_branch (optional): Defaults to 'main'. Only modify this
+                    if there is a specific, known reason to do so.
 
         """
-        super(SermosDeploy, self).__init__()
-        self.access_key = get_access_key(access_key)
+        super().__init__(access_key, base_url, deployment_id)
         self.pkg_name = get_client_pkg_name(pkg_name)
         self.sermos_yaml_filename = sermos_yaml_filename
         self.commit_hash = commit_hash
         self.sermos_yaml = None  # Established later, only on `invoke`
         self.encoded_sermos_yaml = None  # Established later, only on `invoke`
         self.deploy_payload = None  # Established later, only on `invoke`
-        self.deploy_url = deploy_url if deploy_url\
-            else DEFAULT_DEPLOY_URL
         self.deploy_branch = deploy_branch
 
-        # Note: Sermos Cloud's API expects `apikey`
-        self.headers = {
-            'Content-Type': 'application/json',
-            'apikey': self.access_key
-        }
-
     def _set_commit_hash(self):
         """ Retrieve the commit hash of the current git state and set to
             current deployment object.
@@ -120,9 +114,24 @@ class SermosDeploy():
     def get_deployment_status(self):
         """ Info on a specific deployment
         """
-        this_url = self.deploy_url + '/status'
-        r = requests.get(this_url, headers=self.headers)
-        return r
+        resp = self.get(self.services_url)
+        services = resp.json().get('data', {}).get('results', [])
+        status_map = {
+            'data': {
+                'results': []
+            },
+            'message': 'Status of all Deployment Services'
+        }
+        for service in services:
+            status_map['data']['results'].append({
+                'service_id':
+                service['niceId'],
+                'name':
+                service['name'],
+                'status':
+                service['status']
+            })
+        return status_map
 
     def validate_deployment(self, output_file: str = None):
         """ Test rendering sermos.yaml and validate.
@@ -155,7 +164,4 @@ class SermosDeploy():
         self._set_deploy_payload()
 
         # Make request to your environment's endpoint
-        r = requests.post(self.deploy_url,
-                          headers=self.headers,
-                          data=json.dumps(self.deploy_payload))
-        return r
+        return self.post(self.deploy_url, self.deploy_payload)
