@@ -2,6 +2,7 @@
 """
 import re
 from celery.schedules import crontab_parser
+from croniter import croniter
 from marshmallow.validate import OneOf
 from marshmallow.exceptions import ValidationError
 from marshmallow import Schema, fields, EXCLUDE, pre_load, validates_schema
@@ -19,40 +20,31 @@ class ExcludeUnknownSchema(Schema):
 class IntervalScheduleSchema(Schema):
     every = fields.Integer(required=True)
     period = fields.String(
-        requried=True,
+        required=True,
         validate=OneOf(['microseconds', 'seconds', 'minutes', 'hours',
                         'days']))
 
 
-class CrontabValue(fields.String):
-    def __init__(self, regex, *args, **kwargs):
-        self.regex = re.compile(regex)
-        super().__init__(*args, **kwargs)
-
-    def _deserialize(self, value, attr, data, **kwargs):
-        if value is None:
-            return None
-        match = self.regex.fullmatch(value)
-        if match:
-            return value
-        raise ValidationError('Invalid crontab value')
-
-
 class CrontabScheduleSchema(Schema):
-    # Regex expression copied from
-    # https://gist.github.com/harshithjv/c58f0dfce0656cf94c8c
-    minute = CrontabValue(r"\*|[0-5]?\d")
-    hour = CrontabValue(r"\*|[01]?\d|2[0-3]")
-    dayOfWeek = CrontabValue(r"\*|0?[1-9]|[12]\d|3[01]")
-    dayOfMonth = CrontabValue(r"\*|0?[1-9]|1[012]")
-    monthOfYear = CrontabValue(r"\*|[0-6](\-[0-6])?")
+    minute = fields.String(required=True)
+    hour = fields.String(required=True)
+    dayOfWeek = fields.String(required=True)
+    dayOfMonth = fields.String(required=True)
+    monthOfYear = fields.String(required=True)
 
     @validates_schema
     def validate_values(self, data, **kwargs):
         if data['minute'] is None or data['hour'] is None or \
             data['dayOfWeek'] is None or data['dayOfMonth'] is None or\
                 data['monthOfYear'] is None:
-            raise ValidationError("Inavlid crontab value")
+            raise ValidationError("Empty crontab value")
+
+        test_cron_expression = \
+            f"{data['minute']} {data['hour']} {data['dayOfMonth']} " \
+            f"{data['monthOfYear']} {data['dayOfWeek']}"
+
+        if not croniter.is_valid(test_cron_expression):
+            return ValidationError("Invalid crontab value")
 
 
 class Schedule(fields.Dict):
