@@ -155,23 +155,24 @@ class EnvironmentVariablesSchema(Schema):
 
 
 class ServiceRequestsSchema(Schema):
+
     replicaCount = fields.Integer(
         required=False,
         description="Baseline (min) scale of this service to have available.",
         default=1,
         example=1)
     cpuRequest = fields.Float(
-        required=True,
+        required=False,
         description="Requested CPUs to be available for each replica.",
         default=0.5,
         example="0.5")
     memoryRequest = fields.Float(
-        required=True,
+        required=False,
         description="Requested memory (in GB) to be available for each replica.",
         default=0.5,
         example="0.5 (means half of 1 GB)")
     ephemeralStorageRequest = fields.Float(
-        required=True,
+        required=False,
         description="Requested ephemeral storage (in GB) to "
                     "be available for each replica.",
         default=8,
@@ -233,17 +234,30 @@ class SermosImageConfigSchema(NameSchema):
         example="rhoai/sermos:latest")
     repositoryUrl = fields.String(
         required=False,
-        description="The Docker image repository url when using an image "
-        "not built by Sermos. Tag is optional, if excluded, `latest` is used.",
-        example="rhoai/custom-image ; rhoai/custom-image:v0.0.0")
+        description="Deprecated - Use imageUri instead.")
     repositoryUser = fields.String(
         required=False,
-        description="Optional repository username if provided repositoryUrl "
-        "is a private registry.",
-        example="rhoai")
+        description="Deprecated - Use registryUser instead.",)
     repositoryPassword = fields.String(
         required=False,
-        description="Optional repository password if provided repositoryUrl "
+        description="Deprecated - Use registryPassword instead.")
+    imageUri = fields.String(
+        required=False,
+        description="The Docker image uri when using an image not built by "
+        "Sermos. Tag is optional, if excluded, `latest` is used.",
+        example="rhoai/custom-image ; rhoai/custom-image:v0.0.0")
+    registryDomain = fields.String(
+        required=False,
+        description="Registry domain for using a pre-built image.",
+        example="index.docker.io")
+    registryUser = fields.String(
+        required=False,
+        description="Optional registry username if provided registryDomain"
+        "is a private registry.",
+        example="rhoai")
+    registryPassword = fields.String(
+        required=False,
+        description="Optional registry password if provided registryDomain "
         "is a private registry. NOTE: Strongly recommended to use environment "
         "variable interpolation in your sermos.yaml file, do not commit "
         "unencrypted secrets to a git repository. "
