@@ -25,8 +25,8 @@ from urllib.parse import urljoin
 import requests
 from rhodb.redis_conf import RedisConnector
 from sermos.constants import DEFAULT_BASE_URL, PIPELINE_CONFIG_CACHE_KEY, \
-    SCHEDULE_CONFIG_CACHE_KEY, CONFIG_REFRESH_RATE, USING_SERMOS_CLOUD,\
-    LOCAL_DEPLOYMENT_VALUE
+    SCHEDULE_CONFIG_CACHE_KEY, CONFIG_REFRESH_RATE, USING_SERMOS_CLOUD, \
+    LOCAL_DEPLOYMENT_VALUE, DEFAULT_CONFIG_RETRIEVAL_PAGE_SIZE
 from sermos.sermos_yaml import load_sermos_config
 from sermos.pipeline_config_schema import BasePipelineSchema
 from sermos.schedule_config_schema import BaseScheduleSchema
@@ -35,8 +35,8 @@ logger = logging.getLogger(__name__)
 redis_conn = RedisConnector().get_connection()
 
 
-def get_admin_access_key(access_key: Union[str, None] = None,
-                         env_var_name: str = 'SERMOS_ACCESS_KEY'):
+def get_access_key(access_key: Union[str, None] = None,
+                   env_var_name: str = 'SERMOS_ACCESS_KEY'):
     """ Simple helper to get admin server access key in a standard fashion. If
     one is provided, return it back. If not, look in environment for
     `env_var_name`. If that doesn't exist, raise useful error.
@@ -58,7 +58,9 @@ def get_admin_access_key(access_key: Union[str, None] = None,
             "access key in your Sermos Cloud administration console.")
 
 
-def _get_admin_deployment_id(env_var_name: str = 'SERMOS_DEPLOYMENT_ID'):
+# TODO cast to UUID?
+def get_deployment_id(deployment_id: Union[str, None] = None,
+                      env_var_name: str = 'SERMOS_DEPLOYMENT_ID'):
     """ Simple helper to get the deployment id in a standard fashion. Look in
     the environment for `env_var_name`. If that doesn't exist, raise useful
     error.
@@ -67,6 +69,9 @@ def _get_admin_deployment_id(env_var_name: str = 'SERMOS_DEPLOYMENT_ID'):
     so this will simply return 'local' in the event the DEFAULT_BASE_URL is
     set to the LOCAL_DEPLOYMENT_VALUE ('local' by default) in the environment.
     """
+    if deployment_id is not None:
+        return deployment_id
+
     if not USING_SERMOS_CLOUD:
         return LOCAL_DEPLOYMENT_VALUE  # e.g. 'local'
 
@@ -121,7 +126,7 @@ def _generate_api_url(endpoint: str = ''):
     concept in order to use the built in helper functions for retrieving
     pipelines and schedules from an API source.
     """
-    deployment_id = _get_admin_deployment_id()  # From env if None
+    deployment_id = get_deployment_id()  # From env if None
     return urljoin(DEFAULT_BASE_URL, f'deployments/{deployment_id}/{endpoint}')
 
 
@@ -138,8 +143,17 @@ def _retrieve_and_cache_config(key: str,
         return conf
 
     # Ask Sermos Cloud (Note: Sermos Cloud's API expects `apikey`)
-    headers = {'apikey': access_key}
-    r = requests.get(admin_api_endpoint, headers=headers, verify=True)
+    headers = {
+        'apikey': access_key,
+    }
+
+    params = {
+        'page_size': DEFAULT_CONFIG_RETRIEVAL_PAGE_SIZE,
+        'page': 1
+    }
+
+    r = requests.get(admin_api_endpoint, headers=headers, verify=True,
+                     params=params)
 
     data = None
     if r.status_code == 200:
@@ -148,6 +162,22 @@ def _retrieve_and_cache_config(key: str,
         logger.warning(f"Non-200 response retrieving {admin_api_endpoint}: "
                        f"{r.status_code}, {r.reason}")
 
+    # There's a chance we need to request ALL schedule configs from sermos cloud
+    # for the scheduled tasks.  Lets loop and grab all of them.
+    while key == SCHEDULE_CONFIG_CACHE_KEY and \
+            len(data['data']['results']) < data['data']['count']:
+        params['page'] += 1
+        r = requests.get(admin_api_endpoint, headers=headers, verify=True,
+                         params=params)
+        if r.status_code == 200:
+            paginated_data = r.json()
+            data['data']['results'] = data['data']['results'] + \
+                                      paginated_data['data']['results']
+        else:
+            logger.warning(f"Non-200 response retrieving {admin_api_endpoint}: "
+                       f"{r.status_code}, {r.reason}")
+            break
+
     # Cache result
     if data is not None:
         set_json_config_to_redis(key, data, refresh_rate)
@@ -201,7 +231,7 @@ def retrieve_latest_pipeline_config(
     # service for pipeline configuration. If this deployment is set up to
     # cache results, do so.
     cache_key = PIPELINE_CONFIG_CACHE_KEY.format(pipeline_id)
-    access_key = get_admin_access_key(access_key)  # From env if None
+    access_key = get_access_key(access_key)  # From env if None
 
     # Generate pipeline specific API endpoint. If pipeline_id
     # is None, then we're asking for 'all' pipelines.
@@ -249,7 +279,7 @@ def retrieve_latest_schedule_config(access_key: Union[str, None] = None,
         return None
 
     cache_key = SCHEDULE_CONFIG_CACHE_KEY
-    access_key = get_admin_access_key(access_key)  # From env if None
+    access_key = get_access_key(access_key)  # From env if None
 
     api_url = _generate_api_url('scheduled_tasks')
 
@@ -278,7 +308,7 @@ def update_schedule_config(new_schedule_config: dict,
     if not USING_SERMOS_CLOUD:
         return True
 
-    access_key = get_admin_access_key(access_key)  # From env if None
+    access_key = get_access_key(access_key)  # From env if None
     api_url = _generate_api_url('scheduled_tasks')
 
     # Ask Sermos Cloud (Note: Sermos Cloud's API expects `apikey`)
